//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 06/02/02; 14:43:24

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "dicolib.h"
#include "menulib.h"
#include "motys.h"

//---------------------------------------------------------------------------

extern TMenu*Menu;
enum {ME_MAIN,ME_PAUSE,ME_OPTIONS};

//---------------------------------------------------------------------------

extern TDico * Dico;
extern char * lang_str[];

//---------------------------------------------------------------------------

unsigned short Tiles_TILE[] =   {0x0010,0x0010,0x0010,0x0010,0x0010,0x0010,0x0010,0x0010,0x0010,0x0010,0x0010,0xFFF0};
unsigned short Tiles_SQUARE[] = {0xFFF0,0xFFF0,0x8030,0x8030,0x8030,0x8030,0x8030,0x8030,0x8030,0x8030,0xFFF0,0xFFF0};
unsigned short Tiles_ROUND[] =  {0xE0F0,0xC070,0x8030,0x0010,0x0010,0x0010,0x0010,0x0010,0x8030,0xC070,0xE0F0,0xFFF0};
unsigned short Tiles_WRONG[] =  {0x0010,0x0010,0x0010,0x0010,0x0010,0xFFF0,0xFFF0,0x0010,0x0010,0x0010,0x0010,0xFFF0};

//---------------------------------------------------------------------------

char * yesno_str[] = {"No","Yes"};

char msg[50];

char * Plane0;

TGame *Game;

//---------------------------------------------------------------------------


void RefreshScreen(void)
{
	int a,b;
	
	
	PortSet(Plane0,239,127);
	ClrScr();
	
	
	// WORDS TRIED
	
	for (a=0;a<WORD_LENGTH;a++)
	for (b=0;b<TRIES_MAX;b++)
	{
		int posx=(a<<3)+(a<<2);
		int posy=(b<<3)+(b<<2);
		
		unsigned short *addr=NULL;
		switch (Game->tries[b].state[a])
		{
			case TG_TILE : addr=Tiles_TILE;break;
			case TG_SQUARE : addr=Tiles_SQUARE;break;
			case TG_ROUND : addr=Tiles_ROUND;break;
			case TG_WRONG : addr=Tiles_WRONG;break;
		}
		
		Sprite16_OR(posx,posy,12,addr,Plane0);
		DrawChar(posx+2,posy+2,Game->tries[b].word[a],A_NORMAL);
	}
	
	// ACTUAL WORD
	
	for (a=0;a<WORD_LENGTH;a++)
	{
		int posx=(a<<3)+(a<<2);
		
		Sprite16_OR(posx,84,12,a==Game->pos_x?Tiles_SQUARE:Tiles_TILE,Plane0);
		DrawChar(posx+2,84+2,Game->ActualWord[a],A_NORMAL);
	}
	
	// MESSAGE
	
	DrawStr(80,0,msg,A_NORMAL);
	
	
	PortRestore();
	FastCopyScreen(Plane0,LCD_MEM);
	
}
//---------------------------------------------------------------------------

void NewGame(void)
{
	int a,b;
	
	
	strcpy(Game->SolutionStr,GetWord(random(Dico->MAXWORDS)));
	
	for (b=0;b<TRIES_MAX;b++)
	{
		// STRING DELIMITER
		Game->tries[b].word[WORD_LENGTH]=0;
		
		for (a=0;a<WORD_LENGTH;a++)
		{
			Game->tries[b].word[a] = ' ';
			Game->tries[b].state[a] = TG_TILE;
		}
	}
	
	// STRING DELIMITER
	Game->ActualWord[WORD_LENGTH]=0;
	
	for (a=0;a<WORD_LENGTH;a++)
	Game->ActualWord[a] = ' ';
	
	Game->pos_x = 0;
	Game->pos_y = 0;
	
	NewTry();
	
}
//---------------------------------------------------------------------------


void OnBadWord(void)
{
	
	Game->pos_y++;
	
	// LAST TRY -> LOSE
	if (Game->pos_y==TRIES_MAX)
	{
		strcpy(Game->ActualWord,Game->SolutionStr);
		strcpy(msg,"You lose!");
		RefreshScreen();
		strcpy(msg,"");
		ngetchx();
		
		NewGame();
		
	}
	else
	{
		NewTry();
	}
	
	
}
//---------------------------------------------------------------------------

void OnWin(void)
{
	
	sprintf(msg,"You win!");
	RefreshScreen();
	strcpy(msg,"");
	ngetchx();
	
	NewGame();
	
}
//---------------------------------------------------------------------------

void NewTry(void)
{
	int a;
	
	Game->pos_x = 0;
	
	
	for (a=0;a<WORD_LENGTH;a++)
	{
		Game->tries[Game->pos_y].word[a] = '.';
		Game->ActualWord[a] = ' ';
	}
	
	// FIRST TRY
	if (Game->pos_y==0)
	{
		for (a=0;a<WORD_LENGTH;a++)
		Game->KnownLetters[a] = a==0?1:0;
	}
	
	// KNOWN LETTERS
	for (a=0;a<WORD_LENGTH;a++)
	{
		if (Game->KnownLetters[a])
		Game->tries[Game->pos_y].word[a] = Game->SolutionStr[a];
	}
	
	
	
}
//---------------------------------------------------------------------------

void ValidateWord(void)
{
	int a;
	
	
	if (!Game->accept_bad_words)
	Game->word_is_accepted = WordExists(Game->ActualWord);
	else
	Game->word_is_accepted = 1;
	
	SRCalculate(Game->ActualWord,Game->SolutionStr);
	
	strcpy(Game->tries[Game->pos_y].word,Game->ActualWord);
	
	if (Game->squares==WORD_LENGTH)
	{
		OnWin();
	}
	else
	{
		
		// WRONG WORD
		if (!Game->word_is_accepted) 
		{
			for (a=0;a<WORD_LENGTH;a++)
			Game->tries[Game->pos_y].state[a] = TG_WRONG;
		}
		
		OnBadWord();
	}
	
}
//---------------------------------------------------------------------------

void SRCalculate(char * str1, char * str2)
{
	int a,b;
	
	int word_src_used[10];
	int word_dest_used[10];
	
	// CLEAR USED LETTERS
	for (a=0;a<WORD_LENGTH;a++)
	{
		word_src_used[a] = 0;
		word_dest_used[a] = 0;
	}
	
	
	Game->squares = 0;
	Game->rounds = 0;
	
	// Game->squares
	
	for (a=0;a<WORD_LENGTH;a++)
	if (str1[a]==str2[a])
	if (!word_src_used[a] && !word_dest_used[a]) 
	{
		Game->squares++;
		
		Game->tries[Game->pos_y].state[a] = TG_SQUARE;
		
		if (Game->word_is_accepted)
		Game->KnownLetters[a] = 1;
		
		word_src_used[a] = 1;
		word_dest_used[a] = 1;
	}
	
	// Game->rounds
	
	for (a=0;a<WORD_LENGTH;a++)
	for (b=0;b<WORD_LENGTH;b++)
	if (str1[a]==str2[b])
	if (!word_src_used[a] && !word_dest_used[b]) 
	{
		Game->rounds++;
		
		Game->tries[Game->pos_y].state[a] = TG_ROUND;
		
		word_src_used[a] = 1;
		word_dest_used[b] = 1;
	}
	
	
}

//-----------------------------------------------------------------------------

void AddLetter(char letter)
{
	Game->ActualWord[Game->pos_x] = letter;
	Game->pos_x++;
	
	// END OF WORD
	if (Game->pos_x==WORD_LENGTH)
	ValidateWord();
	
}

//-----------------------------------------------------------------------------



// Main Function
void _main(void)
{
	short key;
	
	randomize();
	
	if (!(Plane0=malloc(LCD_SIZE)))
	goto end_prog;
	
	TGame GameA;
	Game=&GameA;
	
	TMenu MenuA;
	Menu = &MenuA;
	
	TDico DicoA;
	Dico=&DicoA;
	
	
	Dico->language = TL_ENGLISH;
	Game->accept_bad_words = 0;
	
	
	for (;;)
	{
		
		MenuNew2();
		MenuAddTitle("Motys");
		MenuAddItem("Play");
		MenuAddItem("Options");
		MenuAddItem("About");
		MenuAddItem("Quit");
		
		MenuDo(ME_MAIN);
		
		if (!Menu->ok) break;
		
		if (Menu->Index==1)
		{
			
			
			if (!DicoLoad())
			continue;
			
			NewGame();
			
			for(;;)
			{
				
				RefreshScreen();
				key = ngetchx();
				
				
				if (key>='A' && key<='Z')
				AddLetter(key);
				else if (key>='a' && key<='z')
				AddLetter(key-'a'+'A');
				
				
				else if (key==KEY_BACKSPACE)
				{
					if (Game->pos_x>0)
					Game->pos_x--;
					
					Game->ActualWord[Game->pos_x] = ' ';
				}
				
				else if (key==KEY_F2)
				{
					NewGame();
				}
				
				else if (key==KEY_APPS)
				{
					off();
				}
				
				else if (key==KEY_ESC)
				{
					MenuNew2();
					MenuAddTitle("Exit Game?");
					MenuAddItem("Yes");
					MenuAddItem("No");
					
					MenuDo(ME_PAUSE);
					
					if (!Menu->ok || Menu->Index==2)
					continue;
					else
					break;
				}
				
				
			}
			
			
		}
		
		else if (Menu->Index==2)
		{
			
			char buffer[50];
			
			for (;;)
			{
				
				MenuNew2();
				MenuAddTitle("Options");
				sprintf(buffer,"Accept bad words : %s",yesno_str[Game->accept_bad_words]);
				MenuAddItem(buffer);
				sprintf(buffer,"Language : %s",lang_str[Dico->language]);
				MenuAddItem(buffer);
				
				
				MenuDo(ME_OPTIONS);
				
				if (!Menu->ok)
				break;
				
				
				if (Menu->Index==1)
				Game->accept_bad_words^=1;
				else if (Menu->Index==2)
				Dico->language^=1;
				
			}
			
		}
		
		else if (Menu->Index==3)
		{
			clrscr();
			FontSetSys(0);
			printf("About \n");
			printf("Motys 1.2 \n");
			printf("Programmed by Olivier Sangala \n");
			printf("http://olivier.sangala.free.fr \n");
			printf("olivier_sangala@hotmail.com \n");
			FontSetSys(1);
			ngetchx();
		}
		
		else
		break;
		
		
	}
	
	end_prog:
	
	if (Plane0)
	free(Plane0);
	
}
